from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def provider_url():
    url = getattr(settings, 'CENTRALAUTH_PROVIDER_URL', None)
    if not url:
        raise ImproperlyConfigured('Centralauth requires CENTRALAUTH_PROVIDER_URL setting.')
    return url.rstrip('/')


PROVIDER_URL = provider_url()
USER_ENDPOINT = '{0}/user/'.format(PROVIDER_URL)
REGISTER_PERMS_ENDPOINT = '{0}/perms/'.format(PROVIDER_URL)
TOKEN_ENDPOINT = '{0}/o/token/'.format(PROVIDER_URL)
AUTHORISATION_ENDPOINT = '{0}/o/authorize/'.format(PROVIDER_URL)
REFRESH_ENDPOINT = '{0}/o/token/'.format(PROVIDER_URL)



# Generated by Django 2.1.3 on 2018-11-14 12:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('provider', '0002_auto_20181107_0932'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='application',
            options={'verbose_name': 'Application', 'verbose_name_plural': 'Applications'},
        ),
        migrations.AlterModelOptions(
            name='applicationpermission',
            options={'ordering': ('application', 'repr'), 'verbose_name': 'Application permission', 'verbose_name_plural': 'Application permissions'},
        ),
        migrations.AlterModelOptions(
            name='applicationpermissiongroup',
            options={'ordering': ('application', 'name'), 'verbose_name': 'Application permission group', 'verbose_name_plural': 'Application permission groups'},
        ),
        migrations.AlterField(
            model_name='applicationpermission',
            name='application',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL, verbose_name='Application'),
        ),
        migrations.AlterField(
            model_name='applicationpermission',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Date (created)'),
        ),
        migrations.AlterField(
            model_name='applicationpermissiongroup',
            name='application',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL, verbose_name='Application'),
        ),
        migrations.AlterField(
            model_name='applicationpermissiongroup',
            name='permissions',
            field=models.ManyToManyField(blank=True, to='provider.ApplicationPermission', verbose_name='Permissions'),
        ),
        migrations.AlterField(
            model_name='applicationuser',
            name='application',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL, verbose_name='Application'),
        ),
        migrations.AlterField(
            model_name='applicationuser',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Date (created)'),
        ),
        migrations.AlterField(
            model_name='applicationuser',
            name='groups',
            field=models.ManyToManyField(blank=True, to='provider.ApplicationPermissionGroup', verbose_name='Groups'),
        ),
        migrations.AlterField(
            model_name='applicationuser',
            name='permissions',
            field=models.ManyToManyField(blank=True, to='provider.ApplicationPermission', verbose_name='Permissions'),
        ),
        migrations.AlterField(
            model_name='applicationuser',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



from django.conf.urls import url

from .views import CallbackView, LoginView


app_name = 'centralauth_client'
urlpatterns = [
    url('^login/$', LoginView.as_view(), name='login'),
    url('^login/callback/$', CallbackView.as_view(), name='login-callback'),
]



from time import time

from django.conf import settings
from django.contrib.auth import logout
from django.shortcuts import redirect
from oauthlib.oauth2.rfc6749.errors import OAuth2Error

from centralauth.compat import MiddlewareMixin, is_authenticated

from . import services


class CentralAuthSyncMiddleware(MiddlewareMixin):
    def process_request(self, request, retries=0):
        if not is_authenticated(request.user):
            return

        backend = request.session.get('_auth_user_backend')
        if not backend == 'centralauth.client.backends.OAuthBackend':
            return

        if retries > 0:
            # We might have triggered a race condition (two refreshes in parallel).
            # We check if we have a "newer" token in the session.
            request.session['centralauth_token'] = request.session.load().get(
                'centralauth_token'
            )

        token = services.load_token(request.session) or {}
        if token.get('expires_at', 0) > time():
            return

        client = services.oauth2_client(token, request.session)
        try:
            ok = services.sync_user(request.user, client)
        except OAuth2Error:
            # An oauth error is raised if the refresh token was already used.
            # This might happen if the refresh is triggered twice.
            # We then retry 2 times.
            if retries < 2:
                return self.process_request(request, retries + 1)

            ok = False

        if not ok or not request.user.is_active:
            logout(request)
            return redirect('{0}?next={1}'.format(settings.LOGIN_URL, request.path))

        return None



import django
from django.utils.http import is_safe_url as django_is_safe_url


try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse  # noqa


try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:

    class MiddlewareMixin:
        pass


def is_authenticated(user):
    if django.VERSION[0:2] < (1, 10):
        return user.is_authenticated()

    return bool(user.is_authenticated)


def is_safe_url(url):
    if django.VERSION[0:2] < (1, 11):
        return django_is_safe_url(url)

    return django_is_safe_url(url, allowed_hosts=None)


def m2m_set_objects(m2m_target, new_set):
    old_objects = set(m2m_target.all())
    new_objects = []
    for obj in new_set:
        if obj in old_objects:
            old_objects.remove(obj)
        else:
            new_objects.append(obj)

    m2m_target.remove(*old_objects)
    m2m_target.add(*new_objects)



# Generated by Django 2.1.2 on 2018-11-07 08:32

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('provider', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('repr', models.CharField(max_length=255, verbose_name='Name')),
                ('codename', models.CharField(max_length=100, verbose_name='Code name')),
                ('app_label', models.CharField(max_length=100, verbose_name='App label')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date created')),
                ('application', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL)),
            ],
            options={
                'verbose_name': 'Application permission',
                'verbose_name_plural': 'Application permissions',
                'ordering': ['application', 'repr'],
            },
        ),
        migrations.CreateModel(
            name='ApplicationPermissionGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Name')),
                ('application', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL)),
                ('permissions', models.ManyToManyField(blank=True, to='provider.ApplicationPermission')),
            ],
            options={
                'verbose_name': 'Application permission group',
                'verbose_name_plural': 'Application permission groups',
                'ordering': ['application', 'name'],
            },
        ),
        migrations.CreateModel(
            name='ApplicationUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='Superuser status')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Name')),
                ('application', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.OAUTH2_PROVIDER_APPLICATION_MODEL)),
                ('groups', models.ManyToManyField(blank=True, to='provider.ApplicationPermissionGroup')),
                ('permissions', models.ManyToManyField(blank=True, to='provider.ApplicationPermission')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Application user',
                'verbose_name_plural': 'Application users',
            },
        ),
        migrations.AlterUniqueTogether(
            name='applicationuser',
            unique_together={('user', 'application')},
        ),
        migrations.AlterUniqueTogether(
            name='applicationpermissiongroup',
            unique_together={('application', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='applicationpermission',
            unique_together={('application', 'codename', 'app_label')},
        ),
    ]



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend

from . import services


User = get_user_model()


class OAuthBackend(ModelBackend):
    """Authenticate user with OAuth access token."""

    def authenticate(self, request, token):
        client = services.oauth2_client(token)
        details = services.user_details(client)
        if not details:
            return None
        try:
            user = User.objects.get(username=details['username'])
        except User.DoesNotExist:
            user = User(username=details['username'])
            user.save()
        services.update_user(user, **details)
        return user

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.urls import include, path

from .views import PermsEndpoint, UserEndpoint


urlpatterns = [
    path('o/', include('centralauth.provider.oauth_urls', namespace='oauth2_provider')),
    path('user/', UserEndpoint.as_view(), name='user'),
    path('perms/', PermsEndpoint.as_view(), name='perms'),
]



# Generated by Django 2.1.2 on 2018-11-20 22:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('provider', '0003_auto_20181114_1350'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='application',
            options={'ordering': ('name',), 'verbose_name': 'Application', 'verbose_name_plural': 'Applications'},
        ),
    ]



from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import ugettext_lazy as _
from oauth2_provider.models import AbstractApplication


User = get_user_model()


class Application(AbstractApplication):
    """Centralauth custom application model."""

    def save(self, *args, **kwargs):
        self.client_type = 'confidential'
        self.authorization_grant_type = 'authorization-code'
        self.skip_authorization = True
        super().save(*args, **kwargs)

    class Meta(AbstractApplication.Meta):
        verbose_name = _('Application')
        verbose_name_plural = _('Applications')
        ordering = ('name',)


class ApplicationPermission(models.Model):
    """Model for holding all permissions available for application."""

    application = models.ForeignKey(
        Application, verbose_name=_('Application'), on_delete=models.CASCADE
    )
    repr = models.CharField(_('Name'), max_length=255)
    codename = models.CharField(_('Code name'), max_length=100)
    app_label = models.CharField(_('App label'), max_length=100)
    date_created = models.DateTimeField(_('Date (created)'), auto_now_add=True)

    class Meta:
        verbose_name = _('Application permission')
        verbose_name_plural = _('Application permissions')
        unique_together = ('application', 'codename', 'app_label')
        ordering = ('application', 'repr')

    def __str__(self):
        return '{0}: {1}'.format(self.application, self.repr)


class ApplicationPermissionGroup(models.Model):
    """Model for for managing groups of permissions.

    Permission groups are not synced with client Group objects.
    In client all permissions are handled on Permission object level.
    """

    name = models.CharField(_('Name'), max_length=255)
    application = models.ForeignKey(
        Application, verbose_name=_('Application'), on_delete=models.CASCADE
    )
    permissions = models.ManyToManyField(
        ApplicationPermission, verbose_name=_('Permissions'), blank=True
    )

    date_created = models.DateTimeField(_('Name'), auto_now_add=True)

    class Meta:
        verbose_name = _('Application permission group')
        verbose_name_plural = _('Application permission groups')
        unique_together = ('application', 'name')
        ordering = ('application', 'name')

    def __str__(self):
        return self.name


class ApplicationUser(models.Model):
    """Model for managing user permissions within application."""

    user = models.ForeignKey(User, verbose_name=_('User'), on_delete=models.CASCADE)
    application = models.ForeignKey(
        Application, verbose_name=_('Application'), on_delete=models.CASCADE
    )

    is_superuser = models.BooleanField(
        _('Superuser status'),
        default=False,
        help_text=_(
            'Designates that this user has all permissions without '
            'explicitly assigning them.'
        ),
    )
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )

    permissions = models.ManyToManyField(
        ApplicationPermission, verbose_name=_('Permissions'), blank=True
    )
    groups = models.ManyToManyField(
        ApplicationPermissionGroup, verbose_name=_('Groups'), blank=True
    )

    date_created = models.DateTimeField(_('Date (created)'), auto_now_add=True)

    class Meta:
        verbose_name = _('Application user')
        verbose_name_plural = _('Application users')
        unique_together = ('user', 'application')

    def __str__(self):
        return '{} ({})'.format(self.user, self.application)

    def get_permissions(self):
        """Combine all user permissions.

        Returns:
            list: list of ids of all user permissions.
        """
        result = set([(perm.app_label, perm.codename) for perm in self.permissions.all()])

        for group in self.groups.all():
            for perm in group.permissions.all():
                result.add((perm.app_label, perm.codename))
        return list(result)



from django.apps import AppConfig
from django.conf import settings
from django.contrib import admin
from django.core.exceptions import ImproperlyConfigured


class ClientConfig(AppConfig):
    name = 'centralauth.client'

    def ready(self):
        if 'centralauth.client.middleware.CentralAuthSyncMiddleware' not in (
            getattr(settings, 'MIDDLEWARE', ()) or getattr(settings, 'MIDDLEWARE_CLASSES', ())
        ):
            raise ImproperlyConfigured(
                'You cannot use CentralAuth without installing '
                'centralauth.client.middleware.CentralAuthSyncMiddleware'
            )

        login_template = getattr(settings, 'CENTRALAUTH_CUSTOM_LOGIN_TEMPLATE', False)
        if login_template:
            admin.site.login_template = (
                login_template
                if not isinstance(login_template, bool)
                else 'centralauth/client/client_login.html'
            )



from django import forms
from django.contrib import admin
from django.forms import models
from django.utils.translation import ugettext_lazy as _
from oauth2_provider.models import (
    get_access_token_model,
    get_application_model,
    get_grant_model,
    get_refresh_token_model,
)

from .models import ApplicationPermissionGroup, ApplicationUser


Application = get_application_model()


# Unregister not used admins.
admin.site.unregister(Application)
admin.site.unregister(get_grant_model())
admin.site.unregister(get_access_token_model())
admin.site.unregister(get_refresh_token_model())


class ApplicationPermissionGroupAdminForm(models.ModelForm):
    users = forms.ModelMultipleChoiceField(
        ApplicationUser.objects.none(),
        widget=admin.widgets.FilteredSelectMultiple(_('Users'), False),
        required=False,
    )

    class Meta:
        widgets = {'permissions': admin.widgets.FilteredSelectMultiple(_('Permissions'), False)}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance and self.instance.pk:
            self.fields[
                'permissions'
            ].queryset = self.instance.application.applicationpermission_set.all()

            self.fields['users'].queryset = self.instance.application.applicationuser_set.all()

            self.initial['users'] = self.instance.applicationuser_set.values_list(
                'pk', flat=True
            )


@admin.register(ApplicationPermissionGroup)
class ApplicationPermissionGroupAdmin(admin.ModelAdmin):
    list_display = ('name', 'application')
    list_filter = ('application',)
    search_fields = ('name', 'applicationuser__user__username')
    form = ApplicationPermissionGroupAdminForm
    fields = ('name', 'application')

    def get_fields(self, request, obj=None):
        if obj:
            return self.fields + (
                'users',
                'permissions',
            )
        return self.fields

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields + ('application',)
        return self.readonly_fields

    def get_inline_instances(self, request, obj=None):
        # Skip inlines in case of add forms (we pre-filter inlines based on instance).
        if obj is None:
            return []
        return super().get_inline_instances(request, obj)

    def save_related(self, request, form, *args, **kwargs):
        if 'users' in form.cleaned_data:
            form.instance.applicationuser_set.set(form.cleaned_data['users'])
        return super().save_related(request, form, *args, **kwargs)


class ApplicationUserAdminForm(models.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance and self.instance.pk:
            self.fields[
                'groups'
            ].queryset = self.instance.application.applicationpermissiongroup_set.all()


@admin.register(ApplicationUser)
class ApplicationUserAdmin(admin.ModelAdmin):
    list_display = (
        'user',
        'application',
    )
    list_filter = ('application',)
    search_fields = ('user__username',)
    form = ApplicationUserAdminForm
    fields = ('user', 'application', 'is_superuser', 'is_staff', 'is_active')

    def get_fields(self, request, obj=None):
        if obj:
            return self.fields + ('groups',)
        return self.fields

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields + ('application', 'user')
        return self.readonly_fields


class ApplicationUserInlineFormset(models.BaseInlineFormSet):
    def get_form_kwargs(self, index):
        kwargs = super().get_form_kwargs(index)
        if self.instance and self.instance.pk:
            kwargs['parent_instance'] = self.instance
        return kwargs


class ApplicationUserInlineForm(models.ModelForm):
    def __init__(self, *args, **kwargs):
        parent_instance = kwargs.pop('parent_instance', None)
        super().__init__(*args, **kwargs)
        if parent_instance:
            self.fields[
                'groups'
            ].queryset = parent_instance.applicationpermissiongroup_set.all()


class ApplicationUserInline(admin.TabularInline):
    model = ApplicationUser
    formset = ApplicationUserInlineFormset
    form = ApplicationUserInlineForm
    exclude = ('permissions',)
    extra = 0


class ApplicationAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['name'].required = True
        self.fields['redirect_uris'].required = True
        self.fields['client_secret'].required = True


@admin.register(Application)
class ApplicationAdmin(admin.ModelAdmin):
    list_display = ('name', 'redirect_uris')
    form = ApplicationAdminForm
    fields = ('name', 'client_id', 'client_secret', 'redirect_uris')
    inlines = [ApplicationUserInline]

    def get_inline_instances(self, request, obj=None):
        if obj is None:
            return []
        return super().get_inline_instances(request, obj)



from django.conf.urls import url
from oauth2_provider import views


app_name = 'oauth2_provider'

urlpatterns = [
    url(r'^authorize/$', views.AuthorizationView.as_view(), name='authorize'),
    url(r'^token/$', views.TokenView.as_view(), name='token'),
    url(r'^revoke_token/$', views.RevokeTokenView.as_view(), name='revoke-token'),
    url(r'^introspect/$', views.IntrospectTokenView.as_view(), name='introspect'),
]



import json
import logging
from time import time

import requests
from django.conf import settings
from django.contrib.auth.models import Permission
from requests_oauthlib import OAuth2Session

from ..compat import m2m_set_objects
from . import constants


log = logging.getLogger('centralauth')


def serialize_perm(perm):
    """Serialize given permission object.

    Returns:
        dict: keys: app_lable, codename, repr.
    """
    return {
        'app_label': perm.content_type.app_label,
        'codename': perm.codename,
        'repr': str(perm),
    }


def register_perms():
    """Register permissions available in the project to the provider."""
    url = constants.REGISTER_PERMS_ENDPOINT
    headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
    data = {
        'client_id': settings.CENTRALAUTH_CLIENT_ID,
        'client_secret': settings.CENTRALAUTH_CLIENT_SECRET,
        'perms': [serialize_perm(perm) for perm in Permission.objects.all()],
    }
    response = requests.post(url, data=json.dumps(data), headers=headers)
    return response


def oauth2_client(token, session=None):
    kwargs = {}
    if session:
        kwargs.update(
            {
                'auto_refresh_url': constants.REFRESH_ENDPOINT,
                'token_updater': lambda token: save_token(session, token),
                'auto_refresh_kwargs': {
                    'client_id': settings.CENTRALAUTH_CLIENT_ID,
                    'client_secret': settings.CENTRALAUTH_CLIENT_SECRET,
                },
            }
        )
    return OAuth2Session(client_id=settings.CENTRALAUTH_CLIENT_ID, token=token, **kwargs)


def save_token(session, token):
    session['centralauth_token'] = token

    # If the session supports saving, we do it asap to ensure other requests "see"
    # the new access token and don't try to use an old refresh/access token.
    if hasattr(session, 'save'):
        session.save()


def load_token(session):
    token = session.get('centralauth_token', None)
    if token and 'expires_at' in token:
        token['expires_in'] = token['expires_at'] - time()
    return token


def sync_user(user, client):
    details = user_details(client)
    if not details:
        return False
    update_user(user, **details)
    return True


def user_details(client):
    try:
        response = client.get(constants.USER_ENDPOINT)
        response.raise_for_status()
    except requests.exceptions.RequestException:
        log.exception('Failed to fetch user details')
        return None
    return response.json()


def update_user(user, **kwargs):
    """Update user attributes and permissions """
    permissions = kwargs.pop('permissions', [])
    allowed_attributes = (
        'first_name',
        'last_name',
        'email',
        'is_active',
        'is_staff',
        'is_superuser',
    )
    for attr, value in kwargs.items():
        if attr in allowed_attributes and hasattr(user, attr):
            setattr(user, attr, value)

    if user.has_usable_password():
        user.set_unusable_password()

    user.save()

    updated_permissions = []
    for permission in permissions:
        updated_permissions.append(
            Permission.objects.get(
                content_type__app_label=permission[0], codename=permission[1]
            )
        )

    m2m_set_objects(user.user_permissions, updated_permissions)



from django.core.management.base import BaseCommand, CommandError
from django.utils import translation
from requests.exceptions import ConnectionError

from centralauth.client.services import register_perms


class Command(BaseCommand):
    help = 'Synchronise project permissions with the centralauth-provider.'

    def handle(self, *args, **options):
        translation.activate('en-us')

        try:
            response = register_perms()
            if response.status_code == 403:
                raise CommandError(
                    'Permissions endpoint returned 403. '
                    'Ensure you have apllication cleint_id and client_secret '
                    'set in your settings and client application with these '
                    'credentials exists on your provider server.'
                )
            response_json = response.json()
        except ConnectionError:
            raise CommandError(
                'Could not connect to the provider perms endpoint. '
                'Check whether your centralauth provider server is running.'
            )

        if not response_json['success']:
            self.stdout.write('Operation failed.')
            return

        self.stdout.write(
            'Operation successful. {0} permissions synced, {1} permissions '
            'created, {2} permissions deleted. Permissions count: {3}.'.format(
                response_json['synced'],
                response_json['created'],
                response_json['deleted'],
                response_json['count'],
            )
        )



from django.utils.translation import ugettext
from oauth2_provider.exceptions import FatalClientError
from oauth2_provider.oauth2_backends import OAuthLibCore
from oauthlib.oauth2 import AccessDeniedError


class CentralauthOAuthBackend(OAuthLibCore):
    def create_authorization_response(self, request, scopes, credentials, allow):
        if (
            not credentials['request']
            .client.applicationuser_set.filter(user=request.user)
            .exists()
        ):
            raise FatalClientError(
                error=AccessDeniedError(
                    ugettext(
                        'Your user account is not configured for the requested application.'
                    )
                )
            )

        return super().create_authorization_response(request, scopes, credentials, allow)



import json

from django.core.exceptions import PermissionDenied
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import View
from oauth2_provider.views.generic import ProtectedResourceView

from .models import Application, ApplicationPermission, ApplicationUser


class UserEndpoint(ProtectedResourceView):
    """Endpoint for accessing user data.

    This endpoint can be accessed only with valid access token in header.
    """

    def get(self, request, *args, **kwargs):
        user = request.user
        token = request.META['HTTP_AUTHORIZATION'].split(' ')[1]
        application = user.oauth2_provider_accesstoken.get(token=token).application
        try:
            app_user = ApplicationUser.objects.get(user=user, application=application)
        except ApplicationUser.DoesNotExist:
            raise PermissionDenied

        return JsonResponse(
            {
                'username': user.username,
                'first_name': user.first_name,
                'last_name': user.last_name,
                'email': user.email,
                'is_superuser': app_user.is_superuser,
                'is_staff': app_user.is_staff,
                'is_active': app_user.is_active,
                'permissions': app_user.get_permissions(),
            }
        )


class PermsEndpoint(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(PermsEndpoint, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        data = json.loads(request.body)
        if not data.get('client_id', None) or not data.get('client_secret', None):
            raise PermissionDenied

        application = Application.objects.filter(client_id=data['client_id']).first()
        if not application or application.client_secret != data['client_secret']:
            raise PermissionDenied

        perms_data = data.get('perms', [])
        counter_created = 0

        old_perms = application.applicationpermission_set.all()
        for perm in perms_data:
            perm_obj, created = ApplicationPermission.objects.get_or_create(
                application=application,
                app_label=perm['app_label'],
                codename=perm['codename'],
                defaults={'repr': perm['repr']},
            )
            old_perms = old_perms.exclude(pk=perm_obj.pk)
            if created:
                counter_created += 1

        counter_deleted = old_perms.count()
        old_perms.delete()

        return JsonResponse(
            {
                'success': True,
                'synced': len(perms_data),
                'created': counter_created,
                'deleted': counter_deleted,
                'count': application.applicationpermission_set.count(),
            }
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['centralauth',
 'centralauth.client',
 'centralauth.client.management',
 'centralauth.client.management.commands',
 'centralauth.provider',
 'centralauth.provider.migrations']

package_data = \
{'': ['*'],
 'centralauth.client': ['locale/de/LC_MESSAGES/*',
                        'templates/centralauth/client/*'],
 'centralauth.provider': ['locale/de/LC_MESSAGES/*']}

install_requires = \
['Django>=2.2',
 'django-oauth-toolkit>=1.5',
 'requests-oauthlib>=1.3',
 'requests>=2.25']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata'], 'docs': ['Sphinx>=3.5']}

setup_kwargs = {
    'name': 'django-centralauth',
    'version': '2.0.0',
    'description': 'App for managing user access and permissions from multiple projects.',
    'long_description': "django-centralauth\n==================\n\n.. image:: https://img.shields.io/pypi/v/django-centralauth.svg\n   :target: https://pypi.org/project/django-centralauth/\n   :alt: Latest Version\n\n.. image:: https://github.com/lenarother/django-centralauth/workflows/Test/badge.svg?branch=master\n   :target: https://github.com/lenarother/django-centralauth/actions?workflow=Test\n   :alt: CI Status\n\n.. image:: https://codecov.io/gh/lenarother/django-centralauth/branch/master/graph/badge.svg\n   :target: https://codecov.io/gh/lenarother/django-centralauth\n   :alt: Coverage Status\n\n.. image:: https://readthedocs.org/projects/django-centralauth/badge/?version=latest\n   :target: https://django-centralauth.readthedocs.io/en/stable/?badge=latest\n   :alt: Documentation Status\n\n\ndjango-centralauth solves the problem of managing user access and permissions\nfrom multiple projects in one central place.\n\n\nFeatures\n--------\n\n* based on OAuth2 standard.\n* provider app to set up your own user-management application.\n* client app for delegating authentication and permissions management to provider.\n\n\nRequirements\n------------\n\ndjango-centralauth supports Python 3 only and requires at least Django 2. and django-oauth-toolkit.\n\n\nPrepare for development\n-----------------------\n\n.. code-block:: shell\n\n    $ poetry install\n\n\nNow you're ready to run the tests:\n\n.. code-block:: shell\n\n    $ poetry run py.test\n\n\nResources\n---------\n\n* `Documentation <https://django-centralauth.readthedocs.io>`_\n* `Bug Tracker <https://github.com/moccu/django-centralauth/issues>`_\n* `Code <https://github.com/moccu/django-centralauth/>`_\n",
    'author': 'Magdalena Rother',
    'author_email': 'rother.magdalena@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/lenarother/django-centralauth',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4',
}


setup(**setup_kwargs)



from django.conf import settings
from django.contrib.auth import authenticate, login
from django.core.exceptions import PermissionDenied
from django.shortcuts import redirect
from django.views.generic.base import View
from requests_oauthlib import OAuth2Session

from ..compat import is_safe_url, reverse
from . import constants, services


OAUTH_STATE_KEY = 'oauth_state'


def get_oauth_redirect_url(request, next_url=None):
    path = reverse('centralauth_client:login-callback')
    if next_url and is_safe_url(next_url):
        path += '?next={}'.format(next_url)
    return request.build_absolute_uri(path)


class LoginView(View):
    """Request authorization code from provider.

    Set next url in session.
    Build authorization code request and redirect to provider server.
    """

    def get(self, *args, **kwargs):
        next_url = self.request.GET.get('next', None)

        oauth = OAuth2Session(
            client_id=settings.CENTRALAUTH_CLIENT_ID,
            redirect_uri=get_oauth_redirect_url(self.request, next_url),
        )
        authorization_url, state = oauth.authorization_url(constants.AUTHORISATION_ENDPOINT)

        self.request.session[OAUTH_STATE_KEY] = state

        return redirect(authorization_url)


class CallbackView(View):
    """Exchange authorization code for access token and authenticate user.

    Authenticate user with access token.
    Redirect to next_url or admin page.
    """

    def get(self, *args, **kwargs):
        grant_code = self.request.GET.get('code', None)
        state = self.request.GET.get('state', None)
        next_url = self.request.GET.get('next', None)

        state_ok = state == self.request.session.get(OAUTH_STATE_KEY, '')
        if not grant_code or not state or not state_ok:
            raise PermissionDenied

        oauth = OAuth2Session(
            client_id=settings.CENTRALAUTH_CLIENT_ID,
            redirect_uri=get_oauth_redirect_url(self.request, next_url),
            state=state,
        )
        token = oauth.fetch_token(
            token_url=constants.TOKEN_ENDPOINT,
            include_client_id=True,
            code=grant_code,
            client_secret=settings.CENTRALAUTH_CLIENT_SECRET,
        )

        user = authenticate(request=self.request, token=token)
        if user is None:
            raise PermissionDenied
        login(self.request, user)
        services.save_token(self.request.session, token)

        if next_url and is_safe_url(next_url):
            return redirect(next_url)
        return redirect(reverse('admin:index'))
